﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Design;
using System.Collections;
using C1.Win.C1FlexGrid;
using System.Windows.Forms.Design;

namespace FlexCheckedListBox_UITypeEditor
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            List<Client> _listClients = new List<Client>();
            _listClients.Add(new Client("CL001", "Collision"));
            _listClients.Add(new Client("CL003", "Collision, Liability"));
            _listClients.Add(new Client("CL004", "Comprehensive, Medical Payments, Uninsured Motorist"));
            c1FlexGrid1.DataSource = _listClients;

            CheckListEditor checkListEditor = new CheckListEditor(new string[] { "Collision", "Comprehensive", "Liability", "Medical Payments", "Uninsured Motorist", "Additional Options" });
            c1FlexGrid1.Cols["CoverageOptions"].Editor = new UITypeEditorControl(checkListEditor, false);
        }
    }

    public class Client
    {
        public Client(string id, string options)
        {
            ID = id;
            CoverageOptions = options;
        }
        public string ID { get; set; }
        public string CoverageOptions { get; set; }
    }

    #region ** CheckListEditor
    //
    // CheckListEditor
    // UITypeEditor that can be used to edit items in CheckedListBox control.
    // Users can check any combination.
    // 
    public class CheckListEditor : UITypeEditor
    {
        // ** fields
        private IWindowsFormsEditorService _edSvc;
        private CheckedListBox _list;
        private string[] _items;
        private bool _cancel;

        // ** ctor
        public CheckListEditor(string[] items)
        {
            // store reference to items being edited
            _items = items;

            // build selector list
            _list = new CheckedListBox();
            _list.BorderStyle = BorderStyle.Fixed3D;
            _list.CheckOnClick = true;
            _list.ThreeDCheckBoxes = false;
            _list.KeyPress += new KeyPressEventHandler(_list_KeyPress);
        }

        // ** overrides
        override public UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext ctx)
        {
            return UITypeEditorEditStyle.DropDown;
        }
        override public object EditValue(ITypeDescriptorContext ctx, IServiceProvider provider, object value)
        {
            // initialize editor service
            _edSvc = (IWindowsFormsEditorService)provider.GetService(typeof(IWindowsFormsEditorService));
            if (_edSvc == null)
                return value;

            // null means no flags are checked
            if (value == null)
                value = 0;

            // populate the list
            _list.Items.Clear();
            string items = value.ToString();
            foreach (string item in _items)
            {
                // add this item with the proper check state
                CheckState check = items.Contains(item) ? CheckState.Checked : CheckState.Unchecked;
                _list.Items.Add(item, check);
            }
            _list.Height = Math.Min(300, (_list.Items.Count + 1) * _list.Font.Height);
            _list.Width = 150;

            // show the list
            _cancel = false;
            _edSvc.DropDownControl(_list);

            // build return value from checked items on the list
            if (!_cancel)
            {
                // build a comma-delimited string with the checked items
                StringBuilder sb = new StringBuilder();
                foreach (object item in _list.CheckedItems)
                {
                    if (sb.Length > 0) sb.Append(", ");
                    sb.Append(item.ToString());
                }

                // convert empty string
                return (sb.Length > 0) ? sb.ToString() : "None/Other";
            }

            // done
            return value;
        }

        // ** event handlers

        // close editor if the user presses enter or escape
        private void _list_KeyPress(object sender, KeyPressEventArgs e)
        {
            switch (e.KeyChar)
            {
                case (char)27:
                    _cancel = true;
                    _edSvc.CloseDropDown();
                    break;
                case (char)13:
                    _edSvc.CloseDropDown();
                    break;
            }
        }
    }
    #endregion

    #region ** UITypeEditorControl

    /// <summary>
    /// UITypeEditorControl
    /// </summary>
    public class UITypeEditorControl :
        ComboBox,
        IServiceProvider,
        IWindowsFormsEditorService
    {
        //===========================================================================
        #region ** fields
        private UITypeEditor _editor;	// UITypeEditor responsible for editing the values
        private Form _form;		// form used to show the drop down
        private Rectangle _bounds;	// cell bounds (used to position control and form)
        private object _value;		// current editor value 
        private bool _dropped;	// whether the drop down was displayed
        private bool _allowTyping;	// whether to allow user to edit the string representation of the value
        #endregion

        //===========================================================================
        #region ** ctor

        public UITypeEditorControl(UITypeEditor editor, bool allowTyping)
        {
            // save ctor parameters
            _editor = editor;
            _allowTyping = allowTyping;

            // initialize combo
            DropDownStyle = ComboBoxStyle.DropDown;
            DrawMode = DrawMode.OwnerDrawFixed;

            // initialize drop down editor
            _form = new Form();
            _form.StartPosition = FormStartPosition.Manual;
            _form.FormBorderStyle = FormBorderStyle.None;
            _form.ShowInTaskbar = false;
            _form.TopLevel = true;
            _form.Deactivate += new EventHandler(_form_Deactivate);
        }
        #endregion

        //===========================================================================
        #region ** IC1EmbeddedEditor

        public void C1EditorInitialize(object value, IDictionary attributes)
        {
            // initialize editor value
            _value = value;
            if (value != null)
            {
                TypeConverter tc = TypeDescriptor.GetConverter(value.GetType());
                try
                {
                    Text = (string)tc.ConvertToString(value);
                }
                catch
                {
                    Text = string.Empty;
                }
            }

            // initialize editor style
            Font = (Font)attributes["Font"];
            BackColor = (Color)attributes["BackColor"];
            ForeColor = (Color)attributes["ForeColor"];

            // we haven't dropped the editor yet
            _dropped = false;
        }
        public object C1EditorGetValue()
        {
            // return value from drop down or from edit area
            return (_dropped) ? _value : Text;
        }
        public UITypeEditorEditStyle C1EditorGetStyle()
        {
            return _editor.GetEditStyle();
        }
        public void C1EditorUpdateBounds(Rectangle rc)
        {
            // store bounds, will apply when showing the form
            _bounds = Parent.RectangleToScreen(rc);

            // if the user can type, position the combo
            if (_allowTyping)
            {
                rc.Inflate(2, 2);
                ItemHeight = Math.Max(3, rc.Height - 6);
                Bounds = rc;
            }
            else // can't type, so hide the control
            {
                Bounds = Rectangle.Empty;
            }
        }

        #endregion

        //===========================================================================
        #region ** overrides

        // if the user can't type, show drop down right away
        override protected void OnEnter(EventArgs e)
        {
            if (!_allowTyping)
                DoDropDown();
        }

        // if the user can type, show drop down when requested
        override protected void OnDropDown(EventArgs e)
        {
            if (_allowTyping)
                DoDropDown();
        }

        // suppress OnLeave event until we're done editing
        override protected void OnLeave(EventArgs e)
        {
            // if we dropped down, wait until we're done editing
            if (_dropped)
                return;

            // we didn't drop down, allow default processing
            base.OnLeave(e);
        }

        // this is done only to avoid annoying beeps from the ComboBox
        override public bool PreProcessMessage(ref Message msg)
        {
            switch (msg.Msg)
            {
                case 0x100: // WM_KEYDOWN:

                    // digest key that was pressed
                    KeyEventArgs e = new KeyEventArgs((Keys)(int)msg.WParam);

                    // if it was a tab or enter, handle internally but don't pass
                    // to stupid base class to avoid annoying beeps.
                    switch (e.KeyCode)
                    {
                        case Keys.Tab:
                        case Keys.Enter:
                        case Keys.Escape:
                            OnKeyDown(e);
                            return true;
                    }
                    break;
            }

            // allow regular processing
            return false;
        }
        #endregion

        //===========================================================================
        #region ** private members

        private void DoDropDown()
        {
            // fire event as usual
            base.OnDropDown(EventArgs.Empty);

            // if this is a popup, hide editing area
            if (_editor.GetEditStyle() == UITypeEditorEditStyle.Modal)
                Bounds = Rectangle.Empty;

            // show editor and get the new value
            _dropped = true;
            _value = _editor.EditValue((IServiceProvider)this, _value);

            // force the drop down to close
            DrawMode = DrawMode.Normal;
            DrawMode = DrawMode.OwnerDrawFixed;
            DroppedDown = false;
            Capture = false;

            // fire OnLeave so grid knows we're done
            base.OnLeave(EventArgs.Empty);
        }

        #endregion

        //===========================================================================
        #region ** event handlers

        // close drop down when form deactivated
        private void _form_Deactivate(object sender, EventArgs e)
        {
            ((IWindowsFormsEditorService)this).CloseDropDown();
        }

        #endregion

        //===========================================================================
        #region ** IServiceProvider interface

        object IServiceProvider.GetService(Type serviceType)
        {
            if (serviceType == typeof(IWindowsFormsEditorService))
                return (IWindowsFormsEditorService)this;
            return null;
        }

        #endregion

        //===========================================================================
        #region ** IWindowsFormsEditorService interface

        void IWindowsFormsEditorService.DropDownControl(Control control)
        {
            // size form
            _form.ClientSize = control.Size;

            // calculate form location to avoid being off the screen
            Point pt = new Point(_bounds.Right - control.Width, _bounds.Bottom);
            Rectangle rc = Screen.GetWorkingArea(this);
            if (_bounds.Bottom + control.Height > rc.Bottom)	// check bottom
                pt.Y = _bounds.Top - _form.Height;
            if (pt.Y < 0) pt.Y = 0;					// check top
            if (_bounds.X + control.Width > rc.Right)		// check right
                pt.X = rc.Right - _form.Width;
            if (pt.X < 0) pt.X = 0;					// check left

            // position form
            _form.Location = pt;

            // add control to form and show it
            _form.Controls.Add(control);
            _form.Show();

            // readjust form size (seems redundant, but isn't...)
            _form.ClientSize = control.Size;

            // wait until user makes a selection
            while (control.Visible)
            {
                Application.DoEvents();
                MsgWaitForMultipleObjects(1, IntPtr.Zero, 0, 5, 255);
            }

            // done
            _form.Hide();
            _form.Controls.Clear();
        }
        void IWindowsFormsEditorService.CloseDropDown()
        {
            _form.Hide();
        }
        DialogResult IWindowsFormsEditorService.ShowDialog(Form dialog)
        {
            // support modal editors
            return dialog.ShowDialog();
        }

        #endregion

        //===========================================================================
        #region ** dll imports

        [System.Runtime.InteropServices.DllImport("User32")]
        private static extern int MsgWaitForMultipleObjects(
            int nCount, IntPtr pHandles, short bWaitAll, int dwMilliseconds, int dwWakeMask);

        #endregion
    }
    #endregion
}
